//   _____
// _| LCG |___________________________________________________________________
// Linear Congruential Method, the "minimal standard generator"
// Park & Miller, 1988, Comm of the ACM, 31(10), pp. 1192-1201

#include <math.h>
#include <limits.h>

#include "lcg.h"

long LCG::quotient = LONG_MAX / 16807L ;
long LCG::reminder = LONG_MAX % 16807L ;

void LCG::Seed (uint32 seed) { seed_val = seed; }

LCG::LCG(uint32 seed) { Seed(seed); }

// returns a random unsigned long integer 
uint32
LCG::lRand()
{
	if ( seed_val <= quotient )
		seed_val = (seed_val * 16807L) % LONG_MAX;
	else
	{
		long high_part = seed_val / quotient;
		long low_part  = seed_val % quotient;

		long test = 16807L * low_part - reminder * high_part;

		if ( test > 0 )
			seed_val = test;
		else
			seed_val = test + LONG_MAX;

	}

	return seed_val;
}

// used to convert long 0..2**31 to double 0.0..1.0
// I would have chosen 2147483648, but I didn't write this!
//
#define L2D (1.0/2147483647.0)

double
LCG::dRand ()
{
	uint32 new_rand;

	new_rand = lRand ();
	return new_rand * L2D;

}

// this is the Info about the LCG
static RNGInfoItem lcgitems[] =
{
	{ "Class", "LCG"},
	{ "Description", "Linear Congruential Generator" },
	{ "Authors", "Park & Miller" },
	{ "Published-In", "Comm of the ACM, 31(10), pp. 1192-1201" }
} ;

#define NITEMS (sizeof(lcgitems)/sizeof(lcgitems[0]))

static RNGInfo lcginfo = { NITEMS, lcgitems } ;

RNGInfo *LCG::info = &lcginfo;

